
; BATT.LSP -  10/5/1986  by Tony Tanzillo, A/E Automation Systems
;
; Batt lisp will draw the proper indication for fiberglass batt
; insulation at any desired (reasonable) width and length.
; Typical applications include showing what partitions are to
; be insulated on a floor plan, insulation in cross sections
; and wall sections, etc.
;
; Additional requirements: (distget)  (from the file INPUT.LSP)
;                          (getsnap)  -included here
;                          (perp)     -    "     "
;                          (echo_off) -    "     "
;                          (echo_on)  -    "     "
;                          (lp)       -    "     "
;                          BATT.DWG   -The batt unit module
;
;
; Add to Your ACAD.LSP file, or use (load"batt")
; Once loaded, It adds the command BATT to AutoCAD.
;
; Usage:
;
; At command prompt enter BATT <CR> and You will be prompted for
; one side of the cavity (the area to be filled with insulation).
; Using a wall (or 2 parallel lines) as an example, point to one side
; of the wall at the point where the insulation is to start. Next,
; You will be prompted for point where the insulation is to end.
; Point to the SAME side of the wall, at the point where the insulation
; is to extend to (the distance, measured from the first point and
; parallel to the wall that the insulation is to fill).
; Next, You will be prompted for the width of the cavity or wall,
; or the batt thickness (default is 4"). Assuming You want the insulation
; to take up the entire width of the cavity, point to the opposite side
; of the wall perpendicular to the last point You specified. If You want
; the insulation to be a particular width regardless of the wall thickness,
; then You can enter this width or, drag it from the last point entered.
; Now, You should see the insulation starting from the first point selected,
; continuing parallel to the wall or cavity and it should terminate close to
; the second point entered. (each batt loop is approximately 2" O.C., which
; is way it will not always be the approximate length specified)
;
; IMPORTANT ! this command requires the presence of the drawing file
; BATT.DWG in the current default directory, or it must previously
; INSERTed into the drawing before issuing BATT. You are free to fool
; around with the batt unit module (BATT.DWG) to come up with your own
; preferred designation. Notice that the actual thickness of the batt is
; slightly less then the one you specify, This is a PEN ALLOWANCE which
; prevents the outer loops from merging with the lines that form the
; wall or cavity being filled. You can modify this unit module to your
; hearts content.
;
;
; All comments and suggestions are most welcome.
;
; Tony Tanzillo, A/E Automation Systems  (70307,2556)


(defun c:BATT()
  (echo_off)
  (setq sp (getsnap "One side of Cavity at starting point" "nea" ))
  (command "line" sp)
  (setq ep (getsnap "To" "nea" ))
  (command "")
  (setq ia (angle sp ep))
  (setq wf
    (/ (setq cw (distget ep "Batt thickness, or opposite side of cavity" 4.0)) 3.5)
  )
  (setq n (fix (/ (distance sp ep ) 2 )))
  (setvar "lastpoint"
    (polar
      (polar  sp (+ ia pi) 2 )
      (perp ia)
      (/ cw 2)
    )
  )
  (setq i 0)
  (command "insert" "batt"
    (polar (lp) ia 2) "1" wf
    (angtos ia (getvar "aunits") (getvar "auprec") )
  )
  (while ( < (setq i (1+ i)) n )
    (command "insert" ""
      (polar (lp) ia 2) "1" wf
      (angtos ia (getvar "aunits") (getvar "auprec") )
    )
  )
  (echo_on)
)


(defun getsnap(p m / n v) ; force point with object snap
  (while (null n)
    (setq n
      (osnap
        (setq v
          (getpoint
            (if (boundp 'v)
              (strcat "\nNot Found, Try Again\n" p ": ")
              (strcat p ": ")
            )
          )
        ) m
      )
    )
  )
)


(defun distget( b p d / t)
  (if
    (null
      (setq t
        (getdist
          (if (boundp 'b)
            b
            (list NIL NIL)
          )
          (strcat "\n"
            (if (boundp 'p)
               p
               "Distance"
            )
            " <"

            (rtos (float d)
               (getvar "lunits")
               (getvar "luprec")
            )
            ">: "
          )
        )
      )
    )
    (float d)
    t
  )
)


(defun lp()
 (getvar "lastpoint")
)

(defun perp(a)
  (+ a (/ pi 2) )
)

(defun echo_off ()
(setq echo (getvar "CMDECHO"))
(setvar "CMDECHO" 0)
)

(defun  echo_on()
(setvar "CMDECHO" echo)
)
