

; CTYAREA.LSP
; A program to calculate the area enclosed by a county boundary polyline
; polylines are assumed to be the only entities on a specified layer
; Created By T. Lloyd Benson
; Republic Research, Inc.
; 801 West Main Street
; Charlottesville, VA 22901
; (804)-296-9747


; ############# DEFINE RIGHT JUSTIFIED COLUMN FUNCTION

; a function to add leading blanks and right justify a text string.

(DEFUN rset (string colwidth)
   (WHILE (< (STRLEN string) colwidth)
     (SETQ string (STRCAT " " string))
   )
   (SETQ string string)
)

; ############# INITIALIZE, GET OUTPUT FILE NAME

(SETVAR "cmdecho" 0)
(SETQ pfx '("IN" "MS"))                                    ; Build a list of
                                                           ; acceptable layer
                                                           ; name prefixes

(SETQ fname (GETSTRING "\nFile to contain area data <CTYAREA.DAT>: "))

(IF (= fname "")                                           ; check for default
  (SETQ fname "CTYAREA.DAT")                               ; file name
)

(IF (OPEN fname "r")                                       ; check to see if
  (PROGN                                                   ; file already
    (INITGET 1 "Yes No")                                   ; exists.
    (SETQ flg (GETKWORD "File exists. Replace (Yes/No)? "))
  )
  (SETQ flg "Yes")                                         ; Else set flag
)

; ############ STEP THROUGH LAYERS AND EXTRACT AREA INFO

(IF (= "Yes" flg) (PROGN
  (SETQ fn (OPEN fname "w"))                               ; Open data file
  (SETQ ly (CDR (ASSOC '2 (TBLNEXT "LAYER" T))))           ; Rewind, get lyr 0

  (PRINC "\nWorking...")

  (WHILE (SETQ ly (CDR (ASSOC '2 (TBLNEXT "LAYER"))))      ; While still layers
    (SETQ ent nil)                                         ; Reset ent variable
    (IF (MEMBER (SUBSTR ly 1 2) pfx) (PROGN                ; check if layer
                                                           ; prefix okay.
      (SETQ ent
        (SSGET "X"                                         ; Create an entity
          (LIST                                            ; filter to
            (CONS '0 "POLYLINE")                           ; match only plines
            (CONS '8 ly)                                   ; on current layer.
          )                                                ; end list
        )                                                  ; end ssget
      )                                                    ; end setq
    ))                                                     ; endif & progn

    (IF ent (PROGN                                         ; if pline found,
      (PRINC (STRCAT "\n" ly))                             ; Display county
      (COMMAND "AREA" "ENTITY" ent)                        ; calc county area
      (SETQ ar (RTOS (GETVAR "area")))                     ; and retrieve it
      (WRITE-LINE (STRCAT ly (RSET ar 10)) fn)             ; & write to file
    ))                                                     ; endif & progn
  )                                                        ; endwhile
  (CLOSE fn)

))                                                         ; ENDIF & ENDPROGN

; ################ DONE, SO SET ALL VARIABLES TO NIL

(SETVAR "cmdecho" 1)
(PRINC "\nProcedure complete.")
(SETQ ly nil
      ent nil
      ename nil
      flg nil
      fn nil
      ar nil
      pr nil
      pfx nil
      rset nil
)
(PRINC)
